import { useCallback, useState } from 'react';
import { request } from '../api/client';
import { ApiEndpoint, ApiEndpointTypes } from '../api/endpoints';
import { Options } from 'ky';
import { toFormData } from '../utilities/helpers';

export type UseApiHookReturn<P extends {}, R extends {}> = [
  (params?: P) => Promise<R>,
  {
    loading: boolean;
    progress: number | undefined;
    error: Error | undefined;
    data: R | undefined;
    reset: () => void;
  },
];

export type UseApiHookOptions = {
  encodeAsFormData?: boolean;
  type?: 'json' | 'blob';
};

export const useApi = <
  E extends ApiEndpoint | string,
  P extends E extends ApiEndpoint ? ApiEndpointTypes[E]['params'] : any,
  R extends E extends ApiEndpoint ? ApiEndpointTypes[E]['response'] : any,
>(
  endpoint: E,
  hookOptions: UseApiHookOptions = { encodeAsFormData: true, type: 'json' },
  requestOptions?: Options,
): UseApiHookReturn<P, R> => {
  const [loading, setLoading] = useState<boolean>(false);
  const [progress, setProgress] = useState<number | undefined>(undefined);
  const [error, setError] = useState<Error | undefined>(undefined);
  const [data, setData] = useState<R | undefined>(undefined);

  const exec = useCallback(
    (params?: P): Promise<R> => {
      setError(undefined);
      setLoading(true);

      let parsedParams: { body: FormData } | { json: P } | undefined;

      if (params) {
        if (hookOptions.encodeAsFormData) {
          parsedParams = { body: toFormData(params) };
        } else {
          parsedParams = { json: params };
        }
      }

      return new Promise((resolve, reject) => {
        const requestInstance = request(endpoint, {
          ...parsedParams,
          ...requestOptions,
          onDownloadProgress: ({ percent }) => setProgress(percent),
        });

        const transform =
          hookOptions.type === 'blob'
            ? requestInstance.blob
            : requestInstance.json;

        transform()
          .then((res) => {
            resolve(res as R);
            setData(res as R);
          })
          .catch((err) => {
            reject(err);
            setError(err);
          })
          .finally(() => {
            setLoading(false);
            setProgress(undefined);
          });
      });
    },
    [endpoint, hookOptions, requestOptions, setLoading, setError, setData],
  );

  const reset = () => {
    setLoading(false);
    setProgress(undefined);
    setError(undefined);
    setData(undefined);
  };

  return [
    exec,
    {
      loading,
      progress,
      error,
      data,
      reset,
    },
  ];
};
