import { useEffect, useMemo, useState } from 'react';
import {
  getNumberOfPages,
  paginate,
  PaginationResultProps,
  PaginationProps,
  DEFAULT_PAGE_SIZE,
} from '../utilities/pagination';
import { clamp, DynamicArrayInput } from '../utilities/helpers';

export type UsePaginationHookControls = {
  paginationData: PaginationResultProps;
  previousPage: () => void;
  nextPage: () => void;
  setPage: (page: number) => void;
};

type UsePaginationHookReturn<T> = [T[], UsePaginationHookControls];

export const usePagination = <T extends any>(
  input: DynamicArrayInput<T>,
  { page: initialPage = 1, pageSize = DEFAULT_PAGE_SIZE }: PaginationProps,
): UsePaginationHookReturn<T> => {
  const [page, setPage] = useState<number>(initialPage);
  const [total, setTotal] = useState<number>(0);

  const items = useMemo(
    () => paginate(input, { page, pageSize }),
    [input, page, pageSize],
  );

  const _setPage = (input: number | 'prev' | 'next') => {
    switch (input) {
      case 'prev':
      case 'next':
        setPage((current) =>
          clamp(input === 'prev' ? current - 1 : current + 1, 1, total),
        );
        return;
    }

    setPage(clamp(input, 1, total));
  };

  useEffect(() => {
    setPage(initialPage);
  }, [initialPage]);

  useEffect(() => {
    setTotal(getNumberOfPages(input, pageSize));
  }, [input, pageSize]);

  return [
    items,
    {
      paginationData: { page, pageSize, total },
      previousPage: () => _setPage('prev'),
      nextPage: () => _setPage('next'),
      setPage: _setPage,
    },
  ];
};
