export type EncryptionKeyParams = {
  iv?: string;
  key?: JsonWebKey;
};

export type EncryptedData<T = any> = {
  data: T;
  iv: string;
  key: JsonWebKey;
};

export const encrypt = async (data: BufferSource): Promise<EncryptedData> => {
  const iv = crypto.getRandomValues(new Uint8Array(64));
  const algorithm: AesGcmParams = {
    name: 'AES-GCM',
    iv,
  };

  const key = await crypto.subtle.generateKey(
    {
      name: 'AES-GCM',
      length: 256,
    },
    true,
    ['encrypt', 'decrypt'],
  );

  const encryptedData = await crypto.subtle.encrypt(algorithm, key, data);
  const exportedKey = await crypto.subtle.exportKey('jwk', key);

  return {
    data: encryptedData,
    iv: iv.toString(),
    key: exportedKey,
  };
};

export const decrypt = async ({
  data,
  iv,
  key,
}: EncryptedData): Promise<BufferSource> => {
  const importedKey = await crypto.subtle.importKey(
    'jwk',
    key,
    { name: 'AES-GCM' },
    true,
    ['encrypt', 'decrypt'],
  );

  const parsedIv = new Uint8Array(iv.split(',').map((value) => Number(value)));
  const algorithm: AesGcmParams = {
    name: 'AES-GCM',
    iv: parsedIv,
  };

  crypto.subtle.decrypt(algorithm, importedKey, data).catch(console.log);

  return await crypto.subtle.decrypt(algorithm, importedKey, data);
};

export const encryptFile = async (file: File): Promise<EncryptedData<File>> => {
  const { data, iv, key } = await encrypt(await file.arrayBuffer());

  const encryptedFile = new File([data], file.name, {
    type: file.type,
    lastModified: file.lastModified,
  });

  return {
    data: encryptedFile,
    iv,
    key,
  };
};

export const decryptFile = async ({
  data,
  iv,
  key,
}: EncryptedData<File>): Promise<File> => {
  const buffer = await data.arrayBuffer();
  const decryptedData = await decrypt({ data: buffer, iv, key });

  return new File([decryptedData], data.name, {
    type: data.type,
    lastModified: data.lastModified,
  });
};
