import {
  AxiomLinkMetadataSerialized,
  AxiomLinkQuery,
  ProcessedFileMetadata,
  StoreType,
} from './files';
import { Buffer } from 'buffer';

export const DEV = process.env.NODE_ENV === 'development';
export const IS_BROWSER = typeof window !== 'undefined';

export const IPFS_GATEWAY = 'https://cloudflare-ipfs.com/ipfs/';
export const AXIOM_BASE_URL = DEV
  ? 'http://localhost:3000/'
  : 'https://axiom.dellos.io/';

export const getPageTitle = (title: string) =>
  `${title} • Axiom | Decentralized File-sharing`;

export const toFormData = <T extends {}>(input: T): FormData => {
  const formData = new FormData();

  Object.entries(input).forEach(([key, value]) =>
    formData.append(key, value as any),
  );

  return formData;
};

export type DynamicArrayInput<T> = T[] | (() => T[]);

export const resolveDynamicArrayInput = <T>(
  input: DynamicArrayInput<T>,
): T[] => {
  if (typeof input === 'function') {
    return input();
  } else {
    return input;
  }
};

export const getDisplayPercent = (value: number): number =>
  Math.floor(value * 100);

export const clamp = (value: number, min: number, max: number): number =>
  Math.min(Math.max(value, min), max);

export const toBase64 = (value: any): string =>
  Buffer.from(JSON.stringify(value)).toString('base64');

export const fromBase64 = <T>(value: string): T =>
  JSON.parse(Buffer.from(value, 'base64').toString());

export const generateIPFSGatewayLink = (cid: string): string => {
  return new URL(cid, IPFS_GATEWAY).toString();
};

export const generateAxiomLink = (
  cid: string,
  { key, iv, storeType }: ProcessedFileMetadata,
): string => {
  const url = new URL(`/download`, AXIOM_BASE_URL);

  url.searchParams.append(AxiomLinkQuery.CID, cid);

  if (key && iv) {
    url.searchParams.append(AxiomLinkQuery.EncryptionKey, toBase64(key));
    url.searchParams.append(AxiomLinkQuery.IV, toBase64(iv));
  }

  url.searchParams.append(AxiomLinkQuery.StoreType, storeType);

  return url.toString();
};

export const parseAxiomLinkMetadata = (
  axiomLink: AxiomLinkMetadataSerialized,
): ProcessedFileMetadata => {
  const fileMetadata: ProcessedFileMetadata = {
    storeType: StoreType.RAW,
  };

  const serializedExportedKey = axiomLink?.[AxiomLinkQuery.EncryptionKey];
  const serializedIV = axiomLink?.[AxiomLinkQuery.IV];
  const serializedStoreType = axiomLink?.[AxiomLinkQuery.StoreType];

  if (serializedExportedKey) {
    fileMetadata.key = fromBase64(serializedExportedKey);
  }

  if (serializedIV) {
    fileMetadata.iv = fromBase64(serializedIV);
  }

  if (serializedStoreType) {
    fileMetadata.storeType = serializedStoreType;
  }

  return fileMetadata;
};

export const invokeClientDownload = (file: File): void => {
  const link = document.createElement('a');
  link.style.display = 'none';
  link.href = URL.createObjectURL(file);
  link.download = file.name;

  document.body.appendChild(link);
  link.click();

  setTimeout(() => {
    URL.revokeObjectURL(link.href);
    link.parentNode?.removeChild(link);
  }, 0);
};
