import React, { useEffect, useState } from 'react';
import { PortalContent } from '../app/components/layout/Portal/PortalContent';
import { FormControl } from '../app/components/controls/FormControl';
import { useRouter } from 'next/router';
import {
  invokeClientDownload,
  generateAxiomLink,
  getDisplayPercent,
  parseAxiomLinkMetadata,
  getPageTitle,
} from '../app/utilities/helpers';
import {
  AxiomLinkMetadataSerialized,
  handleFileDownload,
  IPFSDownloadType,
} from '../app/utilities/files';
import cn from 'classnames';
import {
  DownloadTypeToggle,
  useDownloadType,
} from '../app/components/DownloadTypeToggle';
import { Input } from '../app/components/controls/Input';

const Download: React.FC = () => {
  const { query } = useRouter();
  const urlCid = query?.cid as string | undefined;

  const [link, setLink] = useState<string>('');
  const [loading, setLoading] = useState(false);
  const [progress, setProgress] = useState<number>(0);

  const { downloadType, setDownloadType } = useDownloadType();

  useEffect(() => {
    if (urlCid) {
      setLink(
        generateAxiomLink(
          urlCid,
          parseAxiomLinkMetadata(query as AxiomLinkMetadataSerialized),
        ),
      );
    }
  }, [query, urlCid]);

  const _handleDownload = async () => {
    if (link && !loading) {
      setLoading(true);

      void handleFileDownload(link, downloadType, undefined, {
        onProgress: (percent) => setProgress(getDisplayPercent(percent)),
        onCompleted: (file) => {
          invokeClientDownload(file);
          setLoading(false);
        },
        onError: () => setLoading(false),
      });
    }
  };

  return (
    <PortalContent
      title={getPageTitle('Download')}
      subtitle="Retrieve a file from IPFS"
    >
      <FormControl
        htmlFor="download-input"
        label="Enter a CID, or paste an Axiom link"
      >
        <Input
          aria-label="Enter a CID, or paste an Axiom link"
          id="download-input"
          type="text"
          placeholder="bafkreih4hf5ai35zd ..."
          value={link}
          onChange={({ target: { value } }) => setLink(value)}
        />
      </FormControl>
      <DownloadTypeToggle
        className="mb-4"
        downloadType={downloadType}
        setDownloadType={setDownloadType}
        disabled={loading}
      />
      {loading && (
        <FormControl
          htmlFor="download-progress"
          label={
            downloadType === IPFSDownloadType.Gateway
              ? `Progress ${progress ?? 0}%`
              : 'Please wait, this may take a while...'
          }
        >
          <progress
            id="download-progress"
            className="progress progress-primary w-full mb-3"
            value={
              progress && downloadType === IPFSDownloadType.Gateway
                ? progress
                : undefined
            }
            max="100"
          ></progress>
        </FormControl>
      )}
      <FormControl noBottomMargin>
        <button
          aria-label="Start download"
          className={cn('btn', { loading })}
          disabled={!link}
          onClick={_handleDownload}
        >
          {loading ? 'Downloading...' : 'Download'}
        </button>
      </FormControl>
    </PortalContent>
  );
};

export default Download;
