import React, { useState } from 'react';
import { useApi } from '../app/hooks/useApi';
import { ApiEndpoint } from '../app/api/endpoints';
import FileInput from '../app/components/controls/FileInput';
import cn from 'classnames';
import { PortalContent } from '../app/components/layout/Portal/PortalContent';
import {
  FileOptions,
  processFile,
  STORE_TYPES_MAP,
  StoreType,
} from '../app/utilities/files';
import { Select } from '../app/components/controls/Select';
import { FormControl } from '../app/components/controls/FormControl';
import { Checkbox } from '../app/components/controls/Checkbox';
import vault from '../app/utilities/vault';
import {
  generateAxiomLink,
  generateIPFSGatewayLink,
  getPageTitle,
} from '../app/utilities/helpers';
import { LinkResult } from '../app/components/LinkResult';
import { useRouter } from 'next/router';

const Upload: React.FC = () => {
  const router = useRouter();
  const [file, setFile] = useState<File>();
  const [fileOptions, setFileOptions] = useState<FileOptions>({
    storeType: StoreType.RAW,
  });
  const [links, setLinks] = useState({
    gateway: '',
    axiom: '',
  });

  const [upload, { data, loading, reset: resetFetch }] = useApi(
    ApiEndpoint.ContentAdd,
  );

  const onUploadHandler = async () => {
    if (!file || loading) return;

    const {
      file: processedFile,
      storeType,
      ...encryption
    } = await processFile(file, fileOptions);

    upload({ data: processedFile }).then(({ cid }) => {
      vault.addFile(file, cid, fileOptions, encryption);

      setLinks({
        gateway: generateIPFSGatewayLink(cid),
        axiom: generateAxiomLink(cid, {
          ...encryption,
          storeType,
        }),
      });
    });
  };

  const reset = () => {
    setLinks({ gateway: '', axiom: '' });
    resetFetch();
  };

  return (
    <PortalContent
      title={getPageTitle('Upload')}
      subtitle="Here you can upload a file to the IPFS"
    >
      <FormControl htmlFor="upload-type" label="Select store type">
        <Select
          id="upload-type"
          options={Object.entries(STORE_TYPES_MAP).map(([key, type]) => ({
            value: key,
            label: type,
          }))}
          onChange={(value) => {
            reset();
            setFileOptions((prev) => ({
              ...prev,
              storeType: value as StoreType,
            }));
          }}
        />
      </FormControl>
      <FormControl>
        <Checkbox
          id="upload-encrypt"
          label="Encrypt"
          onChange={(encrypt) =>
            setFileOptions((prev) => ({ ...prev, encrypt }))
          }
        />
        <Checkbox
          id="upload-obfuscate"
          label="Obfuscate Filename"
          onChange={(obfuscateFilename) =>
            setFileOptions((prev) => ({ ...prev, obfuscateFilename }))
          }
          disabled={fileOptions.storeType !== StoreType.WRAPPED}
        />
      </FormControl>
      <FormControl htmlFor="upload-file">
        <FileInput
          id="upload-file"
          onDrop={(acceptedFiles) => setFile(acceptedFiles[0])}
          maxFiles={1}
        />
      </FormControl>
      {loading && (
        <FormControl
          htmlFor="upload-progress"
          label="Please wait..."
          className="mb-8"
        >
          <progress
            id="upload-progress"
            className="progress progress-primary w-full"
          />
        </FormControl>
      )}
      <button
        aria-label="Upload a selected file"
        className={cn('btn btn-block', { loading: loading })}
        onClick={onUploadHandler}
        disabled={!file}
      >
        {loading ? 'Uploading...' : 'Upload'}
      </button>
      {data && (
        <>
          <div className="divider">File Uploaded!</div>
          <FormControl label="Public IPFS gateway link (only raw files)">
            <LinkResult
              value={links.gateway}
              onDownload={() => window.open(links.gateway, '_blank')}
            />
          </FormControl>
          <FormControl label="Axiom link (recommended; necessary for decryption)">
            <LinkResult
              value={links.axiom}
              onDownload={() => router.push(links.axiom)}
            />
          </FormControl>
        </>
      )}
    </PortalContent>
  );
};

export default Upload;
